// Sylvain MARECHAL
// sylvain.marechal1@libertysurf.fr
// Rev. 01/07/2005
using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Xml;


namespace tetris
{
    /// <summary>
    /// Structure definissant un score
    /// </summary>
    struct CScore
    {
        private string m_Nom;
        private int m_Score;
        private DateTime m_Date;

        /// <summary>
        /// Le nom de la personne
        /// </summary>
        public string Nom
        {
            get { return m_Nom; }
            set { m_Nom = value; }
        }

        /// <summary>
        /// la valeur du score
        /// </summary>
        public int Score
        {
            get { return m_Score; }
            set { m_Score = value; }
        }

        /// <summary>
        /// La date a laquelle ce score a ete realise
        /// </summary>
        public DateTime Date
        {
            get { return m_Date; }
            set { m_Date = value; }
        }

        /// <summary>
        /// Cette classe implemente l'interface IComparer afin de comparer
        /// 2 scores, ce qui permet l'utilisation de score
        /// </summary>
        public class CmpScore : IComparer
        {
            int  IComparer.Compare(object x, object y)
            {
 	            CScore sx = (CScore)x;
                CScore sy = (CScore)y;
                return -(sx.Score - sy.Score);
            }
        }

    }

    /// <summary>
    /// Cette classe manipule l'ensemble des scores et permet
    /// la lecture et l'ecriture des scores dans in fichier xml
    /// </summary>
    class CScores
    {
        static private string ms_FileName = "tetris.xml";
        static private int ms_maxScores = 10;
        private ArrayList m_tabScores = new ArrayList();

        /// <summary>
        /// Le constructeur lit le fichier et initialise 
        /// le tableau de scores
        /// </summary>
        public CScores()
        {
            Read();
            CScore.CmpScore cmps = new CScore.CmpScore();
            m_tabScores.Sort(cmps);
        }

        /// <summary>
        /// Renvoie le tableau de scores 
        /// </summary>
        public ArrayList Scores
        {
            get { return m_tabScores; }
        }

        /// <summary>
        /// Ajoute un score ds le tableau et met a jour le fichier
        /// de scores
        /// </summary>
        /// <param name="nom">Nom du joueur</param>
        /// <param name="score">Score</param>
        /// <param name="date">Date</param>
        public void Add(String nom, int score, DateTime date)
        {
            InternalAdd(nom, score, date);
            Write();
        }

        /// <summary>
        /// Ajoute un score ds le tableau 
        /// </summary>
        /// <param name="nom">Nom du joueur</param>
        /// <param name="score">Score</param>
        /// <param name="date">Date</param>
        private void InternalAdd(String nom, int score, DateTime date)
        {
            if (nom != "" && score != 0 && date != DateTime.MinValue)
            {
                CScore s = new CScore();
                s.Nom = nom;
                s.Score = score;
                s.Date = date;
                m_tabScores.Add(s);
                CScore.CmpScore cmps = new CScore.CmpScore();
                m_tabScores.Sort(cmps);
            }
        }

        /// <summary>
        /// Renvoie true si le score passe en argument fait partie des 
        /// "ms_maxScores" meilleurs scores
        /// </summary>
        /// <param name="score"></param>
        /// <returns></returns>
        public bool IsBest(int score )
        {
            int maxScores = ms_maxScores;
            if( maxScores <= 0 ) maxScores = 1;
            if (m_tabScores.Count < maxScores ) return true;
            CScore cs = (CScore)m_tabScores[maxScores-1];
            if (cs.Score < score) return true;
            return false;
        }

        /// <summary>
        /// Ecrit le contenu du tableau dans le fichier xml
        /// Seuls les "ms_maxScores" sont stockes
        /// </summary>
        private void Write()
        {
            int nbElemWritten = 0;
            XmlTextWriter xtw = new XmlTextWriter(ms_FileName, Encoding.UTF8);
            xtw.Formatting = Formatting.Indented;
            xtw.WriteStartDocument(true);
            xtw.WriteStartElement("scores");
            foreach (CScore sc in m_tabScores)
            {
                xtw.WriteStartElement("score");
                xtw.WriteElementString("nom", sc.Nom);
                xtw.WriteElementString("valeur", sc.Score.ToString() );
                xtw.WriteElementString("date", sc.Date.ToString("g") );
                xtw.WriteEndElement();
                nbElemWritten++;
                if (nbElemWritten == ms_maxScores) break;
            }
            xtw.WriteEndElement();
            xtw.Flush();
            xtw.Close();
        }

        /// <summary>
        /// Lit le fichier xml et ajoute les scores ds le tableau
        /// </summary>
        private void Read()
        {
            String nom = "";
            int score = 0;
            DateTime date = DateTime.MinValue ;
            bool fWaitValues = false;
            string EltName = "";
            try
            {
                XmlTextReader xtr = new XmlTextReader(ms_FileName);
                while (xtr.Read())
                {
                    if (xtr.NodeType == XmlNodeType.Element)
                    {
                        EltName = xtr.Name;
                        if (EltName == "score")
                        {
                            fWaitValues = true;
                        }
                    }
                    else if (xtr.NodeType == XmlNodeType.Text && fWaitValues)
                    {
                        string EltValue = xtr.Value;
                        if (EltName == "nom")
                        {
                            nom = EltValue;
                        }
                        else if (EltName == "valeur")
                        {
                            score = int.Parse(EltValue);
                        }
                        else if (EltName == "date")
                        {
                            date = DateTime.Parse(EltValue);
                        }
                    }
                    if (xtr.NodeType == XmlNodeType.EndElement)
                    {
                        EltName = xtr.Name;
                        if (EltName == "score")
                        {
                            if (nom != "" && score != 0 && date != DateTime.MinValue)
                            {
                                InternalAdd(nom, score, date);
                            }
                            nom = "";
                            score = 0;
                            date = DateTime.MinValue;
                            fWaitValues = false;
                        }
                    }
                }
                xtr.Close();
            }
            catch( System.IO.FileNotFoundException /*f*/ )
            {
            }
        }

    }
}
